/*
 * Copyright 2005-2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jatva.iterators.lang;

public class NeighbourhoodShortIterator
  extends    AbstractShortIterator
  implements IShortIterator
{
  protected short point = 0;
  protected int offset = 0;
  protected int index  = 0;
  
  public NeighbourhoodShortIterator(short point, int offset)
  {
    int m = Math.max(Short.MAX_VALUE - point, point - Short.MIN_VALUE);
    if(offset > m) offset = m;
    if(offset < 0) offset = 0;
     
    this.point  = point;
    this.offset = offset;
  }

  public NeighbourhoodShortIterator(short point)
  {
    this(point, 0);
  }
  
  public NeighbourhoodShortIterator(NeighbourhoodShortIterator r)
  {
    this.point  = r.point;
    this.offset = r.offset;
    this.index  = r.index;
  }

  public Object clone()
  {
    return new NeighbourhoodShortIterator(this);
  }
  
  public boolean equals(Object o)
  {
    if(!(o instanceof NeighbourhoodShortIterator))
      return false;

    NeighbourhoodShortIterator obj = (NeighbourhoodShortIterator)o;

    return    index  == obj.index 
           && point  == obj.point
           && offset == obj.offset;
  }
  
  public short value()
  {
    return (short)(point + ((index%2 == 0)?(index/2):(-(index+1)/2)));
  }
  
  public boolean stop()
  {
    return index > 2*offset;
  }
  
  public void shift()
  {
    index++;
    
    if(      index%2 == 0
          && point > Short.MAX_VALUE - index/2
       ||    index%2 == 1
          && point < Short.MIN_VALUE + (index+1)/2
      )
      index++;
  }

  public int hashCode ()
  {
    return index + 3*point + 11*offset;
  }
}
