/*
 * Copyright 2005-2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jatva.lang;

import jatva.iterators.lang.IObjectIterator;
import jatva.util.IdentityList;
import jatva.util.WeakIdentitySet;

public class MediatorSet
{
  WeakIdentitySet set      = new WeakIdentitySet();
  MediatorSet     parent   = null;
  IdentityList    children = new IdentityList();
  
  ModelClass      mclass   = null;

  public MediatorSet(ModelClass c, MediatorSet t)
  {
    mclass = c;
    parent = t;
    if(t != null)
    {
      t.addChild(this);
    }
  }
  
  public ModelObject add(ModelObject o, Object[] impls)
  {
    ModelObject res = find(impls, null, true);
    
    if(res == null) set.add(o);
      
    return res;
  }
  
  public void add(ModelObject o)
  {
    if(o == null) return; 
      
    set.add(o);
  }

  public void remove(ModelObject o)
  {
    if(o != null) set.remove(o);
  }
  
  public void addChild(MediatorSet s)
  {
    children.addLast(s);
  }

  public ModelObject find(Object[] implFields)
  {
    IObjectIterator i = set.iterator();
    for(; !i.stop(); i.shift())
    {
      if(((ModelObject)i.value())._ts_compareImplementationFields(implFields))
        return (ModelObject)i.value();
    }

    return null;
  }
  
  public ModelObject find(Object[] impls, MediatorSet ignore, boolean goUp)
  {
    ModelObject r1 = null;
    
    IObjectIterator i = set.iterator();
    for(; !i.stop(); i.shift())
    {
      ModelObject o = (ModelObject)i.value();
      if(o._ts_compareImplementationFields(impls))
      {
        r1 = o;
        break;
      }
    }

    ModelObject r2 = null;
    
    for(int j = 0; j < children.size(); j++)
    {
      MediatorSet c = (MediatorSet)(children.get(j));
      
      if( ignore != c )
      {
        r2 = c.find( impls, null, false );
           
        if( r1 != null && r2 != null )
        {
          throw new ConflictingMediatorObjectsException(r1, r2, impls);
        }
        else if(r2 != null)
          r1 = r2;
       }
    }

    if( goUp && parent != null )
    {
       r2 = parent.find( impls, this, true );
       if( r2 != null )
       {
         throw new ConflictingMediatorObjectsException(r2, mclass._ts_name, impls);
       }
    }

    return r1;
  }
  
}
