/*
 * Copyright 2005-2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jatva.lang;

import jatva.util.Utils;

/**
 *
 * @author Victor Kuliamin
 *
 */
public class PostconditionViolationException extends SpecificationViolationException
{
    private static final String info = "Postcondition violation";

    public PostconditionViolationException( String p_class, String p_method )
    {
        super( info
             , new String[]{ "class", "method" }
             , new String[]{ p_class, p_method }
             );
    }

    public PostconditionViolationException( String p_class, String p_method, String comment )
    {
        super( Utils.appendCommentToFailure( info, comment )
             , new String[]{ "class", "method" }
             , new String[]{ p_class, p_method }
             );
    }

    public PostconditionViolationException( ModelObject object, String p_method )
    {
        super( info
             , new String[]{ "object", "method" }
             , new String[]{ formatModelObject( object ), p_method }
             );
    }

    public PostconditionViolationException( ModelObject object, String p_method, String comment )
    {
        super( Utils.appendCommentToFailure( info, comment )
             , new String[]{ "object", "method" }
             , new String[]{ formatModelObject( object ), p_method }
             );
    }
}
