/*
 * Copyright 2005-2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jatva.lang.synchronization;

import jatva.lang.ModelObject;
import jatva.lang.annotation.JTSInternal;
import jatva.lang.annotation.MediatorClass;
import jatva.lang.annotation.ScenarioClass;

import java.lang.reflect.Field;

import com.unitesk.utils.predicates.Predicate;

/**
 * @author <a href="mailto:monakhov@ispras.ru">Alexander Monakhov</a>
 */
public class JTSFieldSkipFilter implements Predicate<Field>
{
    public static final JTSFieldSkipFilter instance  = new JTSFieldSkipFilter();

    private JTSFieldSkipFilter(){}

    @Override
    public boolean holdsOn( Field f )
    {
        // filter out all jts internal fields
        if( f.getAnnotation( JTSInternal.class ) != null )
            return true;

        Class<?> decl_class = f.getDeclaringClass();

        if( decl_class == Throwable.class )
        {
            String field_name = f.getName();
            return    field_name.equals( "stackTrace" )
                   || field_name.equals( "backtrace" )
                 ;
        }

        // filter out all fields declared in mediator classes
        if( decl_class.getAnnotation( MediatorClass.class ) != null )
            return true;

        // filter out all fields of scenario class that are not
        // references to objects of specification classes
        if( decl_class.getAnnotation( ScenarioClass.class ) != null )
        {
            return !(ModelObject.class.isAssignableFrom( f.getType() ));
        }

        return false;
    }
}
