/*
 * Copyright 2005-2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jatva.tracer.impl;

//=============================================================================

import jatva.tracer.TracerException;

import java.io.IOException;
import java.io.Writer;
import java.util.Properties;

//=============================================================================

/** 
 * This class implements writing trace messages to <code>java.io.Writer</code>.
 *
 * @author  <a href="mailto:barancev@ispras.ru">Alexei Barantsev</a>
 * @version 1.0
 */
public class WriteToWriter extends AbstractWriter {

    private Writer writer;

    /** 
     * Creates a new instance of <code>WriteToWriter</code>.
     * 
     * @param writer The trace destination.
     */
    public WriteToWriter(Writer writer) {
        this.writer = writer;
    }

    public void configure(Properties writerProperties) {
        throw new TracerException("This trace writer can not be configured this way");
    }
    
    public void sendMessage(StringBuffer message) {
        try {
            synchronized (writer) {
                writer.write(message.toString());
                writer.flush(); // Needs optimization!
            }

        } catch (IOException e) {
            throw new TracerException("Could not write to writer " + writer, e);
        }
    }

}
//=============================================================================
// End of file WriteToWriter.java
